/*****************************************************************************
 *
 * FILE:	action.c
 * DESCRIPTION:	perfom action according to option
 * DATE:	Wed, Jun 13 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2001 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: action.c,v 1.9 2001/07/09 15:47:28 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wkf.h>
#include "mailtools.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/
#define	SUFFIX_HTML	".html"
#define	DOCTYPE		"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
#define	CONTENTTYPE	"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=EUC-JP\">\n"



/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static void	printBody00(Mail *, bool_t);
static void	printHeader(MailHeader *, FILE *, bool_t);
static void	printBodyInfo(MailHeader *, FILE *);
static void	printMultiPartInfo(MultiPart *, FILE *);
static void	printContentInfo(ContentType *, FILE *);
static void	extractBodyOfMultiPart(MultiPart *);


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	actionSplit
 * Description:		split multipart mail
 * Arguments:		Mail pointer
 * Return Value:	none
 */
void
actionSplit(mail)
	register Mail *	mail;
{
  MultiPart *		multipart = mail->multipart;
  register FILE *	fout	  = NULL;
  char			buf[BUFSIZ];

  /* body */
  printBody00(mail, FALSE);

  /* multipart */
  for (; multipart != NULL; multipart = multipart->next) {
    fout = fopen(multipart->filename, "w");
    if (fout == NULL) {
      error();
    }
    printHeader(mail->header, fout, FALSE);
    printMultiPartInfo(multipart, fout);
    fputc(LF, fout);
    memset(buf, 0, sizeof(buf));
    while (fgets(buf, sizeof(buf), multipart->ftmp) != NULL) {
      fputs(buf, fout);
      memset(buf, 0, sizeof(buf));
    }
    fclose(fout);
  }
}

/*
 * Function Name:	actionExtract
 * Description:		split multipart mail and decode MIME encoding
 * Arguments:		Mail pointer
 * Return Value:	none
 */
void
actionExtract(mail)
	register Mail *	mail;
{
  /* body */
  printBody00(mail, TRUE);

  /* multipart */
  extractBodyOfMultiPart(mail->multipart);
}

/*
 * Function Name:	actionHTML
 * Description:		split multipart mail, decode MIME encoding and
 *			make HTML files
 * Arguments:		Mail pointer
 * Return Value:	none
 */
void
actionHTML(mail)
	register Mail *	mail;
{
  MailBody *		body	  = mail->body;
  MultiPart *		multipart = NULL;
  String		tagFile	  = NULL;
  String		htmlFile  = NULL;
  size_t		len;
  register FILE *	fout	  = NULL;
  char			buf[BUFSIZ];
  conv_t		ret;

  /* make an outputed HTML filename */
  len = strlen(mail->filename) + length(SUFFIX_HTML) + 1;
  htmlFile = (String)calloc(len, sizeof(char));
  if (htmlFile == NULL) {
    error();
  }
  (void)sprintf(htmlFile, "%s%s", mail->filename, SUFFIX_HTML);
  /**/
  fout = fopen(htmlFile, "w");
  if (fout == NULL) {
    error();
  }

  fputs(DOCTYPE, fout);
  fputs("<html lang=\"ja\">\n", fout);
  fputs("<head>\n", fout);
  fputs(CONTENTTYPE, fout);
  fputs("<title>", fout);
  if (mail->header->subject) {
    ret = wkfDecodeMimeString(mail->header->subject, buf, sizeof(buf), KC_EUC);
    htmlPuts(ret == CONV_OK ? buf : mail->header->subject, fout);
  }
  else {
    fputs("No subject", fout);
  }
  fputs("</title>\n</head>\n<body lang=\"ja\" bgcolor=\"#ffffff\">\n", fout);
  htmlPrintHeader(mail->header, fout);
  fputs("<p>\n", fout);
  memset(buf, 0, sizeof(buf));
  while (fgets(buf, sizeof(buf), body->ftmp) != NULL) {
    htmlPuts(buf, fout);
    memset(buf, 0, sizeof(buf));
  }
  fputs("</p>\n", fout);

  /* multipart */
  extractBodyOfMultiPart(mail->multipart);

  if (mail->multipart) {
    fputs("<div>\n<ol>\n", fout);
    for (multipart = mail->multipart; multipart != NULL;
	 multipart = multipart->next) {
      String			name = NULL;
      ContentType *		ctyp = multipart->content_type;
      ContentDisposition *	cdsp = multipart->content_disposition;

      if (ctyp && wkfDecodeMimeString(ctyp->name, buf, sizeof(buf),
				      KC_EUC) != CONV_ERR) {
	name = strdup(buf);
      }
      fputs("<li>", fout);
      if(ctyp && ctyp->media_type) {
	if (strcmp(ctyp->media_type, "application/msword") == 0) {
	  fprintf(fout, "<img src=\"icons/word.gif\" alt=\"[%s]\" "
			"width=\"20\" height=\"22\" "
			"title=\"Microsoft Word\">", ctyp->media_type);
	}
	else if (strcmp(ctyp->media_type, "application/excel") == 0) {
	  fprintf(fout, "<img src=\"icons/excel.gif\" alt=\"[%s]\" "
			"width=\"20\" height=\"22\" "
			"title=\"Microsoft Excel\">", ctyp->media_type);
	}
	else if (strcmp(ctyp->media_type, "application/powerpoint") == 0) {
	  fprintf(fout, "<img src=\"icons/powerpoint.gif\" alt=\"[%s]\" "
			"width=\"20\" height=\"22\" "
			"title=\"Microsoft PowerPoint\">", ctyp->media_type);
	}
	else if (strcmp(ctyp->media_type, "text/html") == 0) {
	  fprintf(fout, "<img src=\"icons/html.gif\" alt=\"[%s]\" "
			"width=\"20\" height=\"22\" "
			"title=\"HTML Document\">", ctyp->media_type);
	}
	else {
	  fprintf(fout, "<img src=\"icons/plain.gif\" alt=\"[%s]\" "
			"width=\"20\" height=\"22\" "
			"title=\"Attached Document\">", ctyp->media_type);
	}
      }
      fputs("<a href=\"", fout);
      if (cdsp) {
	ret = wkfMimeStringToHexString(cdsp->filename, buf, sizeof(buf));
	htmlPutsAnchor(ret == CONV_OK ? buf : multipart->filename, fout);
      }
      else {
	htmlPutsAnchor(multipart->filename, fout);
      }
      fputs("\">", fout);
      ret =  wkfDecodeMimeString(mail->header->subject, buf, sizeof(buf),
				 KC_EUC),
      fputs(name ? name : ret == CONV_OK ? buf : mail->header->subject, fout);
      fputs("</a></li>\n", fout);
      if (name) {
	free((String)name);
      }
    }
    fputs("</ol>\n</div>\n", fout);
  }
  fputs("</body>\n</html>\n", fout);
  fclose(fout);

  /* make an outputed tag filename */
  if (mail->option == OPT_FILTER) {
    tagFile = (String)calloc(len, sizeof(char));
    if (tagFile == NULL) {
      error();
    }
    (void)sprintf(tagFile, "%s%s", TAG_PREFIX, mail->filename);
    /**/
    fout = fopen(tagFile, "w");
    if (fout == NULL) {
      error();
    }

    fputs("<tr><td><font color=\"#993399\">[", fout);
    fputs(mail->filename, fout);
    fputs("]</font>&nbsp;</td>", fout);
    fputs("<td><font color=\"#009900\">", fout);
    htmlPuts(mail->header->date, fout);
    fputs("</font></td></tr>", fout);
    fputs("<tr><td></td><td><font color=\"#ff0000\">", fout);
    htmlPuts(mail->header->from, fout);
    fputs("</font></td></tr>", fout);
    fputs("<tr><td></td><td><a href=\"", fout);
    fputs(htmlFile, fout);
    fputs("\">", fout);
    if (mail->header->subject) {
      ret = wkfDecodeMimeString(mail->header->subject, buf, sizeof(buf),
				KC_EUC);
      htmlPuts(ret == CONV_OK ? buf : mail->header->subject, fout);
    }
    else {
      fputs("No subject", fout);
    }
    fputs("</a></td></tr>\n", fout);
    fputs("<tr><td colspan=\"2\"><hr></td></tr>\n", fout);
    fclose(fout);
    free((String)tagFile);
  }
  free((String)htmlFile);
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	printBody00
 * Description:		print body to file
 * Arguments:		Mail pointer, decode flag
 * Return Value:	none
 */
static void
printBody00(mail, f_decode)
	Mail *	mail;
	bool_t	f_decode;
{
  MailBody *		body = mail->body;
  register FILE *	fout = NULL;
  char			buf[BUFSIZ];

  fout = fopen(body->filename, "w");
  if (fout == NULL) {
    error();
  }
  printHeader(mail->header, fout, f_decode);
  printBodyInfo(mail->header, fout);
  fputc(LF, fout);
  memset(buf, 0, sizeof(buf));
  while (fgets(buf, sizeof(buf), body->ftmp) != NULL) {
    fputs(buf, fout);
    memset(buf, 0, sizeof(buf));
  }
  fclose(fout);
}

/*
 * Function Name:	printHeader
 * Description:		print header fields to file
 * Arguments:		MailHeader pointer, output FILE pointer, decode flag
 * Return Value:	none
 */
static void
printHeader(header, fout, f_decode)
	MailHeader *	header;
	FILE *		fout;
	bool_t		f_decode;
{
  char		buf[BUFSIZ];
  conv_t	ret;

  if (header->date) {
    fprintf(fout, "Date: %s\n", header->date);
  }
  if (header->from) {
    ret = wkfDecodeMimeString(header->from, buf, sizeof(buf), KC_JIS);
    fprintf(fout, "From: %s\n",
	    f_decode && ret == CONV_OK ? buf : header->from);
  }
  if (header->to) {
    ret = wkfDecodeMimeString(header->to, buf, sizeof(buf), KC_JIS);
    fprintf(fout, "To: %s\n",
	    f_decode && ret == CONV_OK ? buf : header->to);
  }
  if (header->cc) {
    ret = wkfDecodeMimeString(header->cc, buf, sizeof(buf), KC_JIS);
    fprintf(fout, "Cc: %s\n",
	    f_decode && ret == CONV_OK ? buf : header->cc);
  }
  if (header->bcc) {
    ret = wkfDecodeMimeString(header->bcc, buf, sizeof(buf), KC_JIS);
    fprintf(fout, "Bcc: %s\n",
	    f_decode && ret == CONV_OK ? buf : header->bcc);
  }
  if (header->message_id) {
    fprintf(fout, "Message-ID: %s\n", header->message_id);
  }
  if (header->subject) {
    ret = wkfDecodeMimeString(header->subject, buf, sizeof(buf), KC_JIS);
    fprintf(fout, "Subject: %s\n",
	    f_decode && ret == CONV_OK ? buf : header->subject);
  }
  if (header->mime_version) {
    fprintf(fout, "MIME-Version: %s\n", header->mime_version);
  }
}

/*
 * Function Name:	printBodyInfo
 * Description:		print MIME header fields of mail body to file
 * Arguments:		MailHeader pointer, output FILE pointer
 * Return Value:	none
 */
static void
printBodyInfo(header, fout)
	MailHeader *	header;
	FILE *		fout;
{
  printContentInfo(header->content_type, fout);
  if (header->content_transfer_encoding) {
    fprintf(fout, "Content-Transfer-Encoding: %s\n",
	    header->content_transfer_encoding);
  }
  if (header->content_id) {
    fprintf(fout, "Content-ID: %s\n", header->content_id);
  }
  if (header->content_description) {
    fprintf(fout, "Content-Description: %s\n", header->content_description);
  }
}

/*
 * Function Name:	printMultiPartInfo
 * Description:		print MIME header fields of multipart to file
 * Arguments:		MultiPart pointer, output FILE pointer
 * Return Value:	none
 */
static void
printMultiPartInfo(multipart, fout)
	MultiPart *	multipart;
	FILE *		fout;
{
  printContentInfo(multipart->content_type, fout);
  if (multipart->content_transfer_encoding) {
    fprintf(fout, "Content-Transfer-Encoding: %s\n",
	    multipart->content_transfer_encoding);
  }
  if (multipart->content_disposition) {
    ContentDisposition *	cdsp = multipart->content_disposition;

    fputs("Content-Disposition:", fout);
    if (cdsp->disposition) {
      fprintf(fout, " %s", cdsp->disposition);
    }
    if (cdsp->filename) {
      fprintf(fout, "; filename=\"%s\"", cdsp->filename);
    }
    if (cdsp->creation_date) {
      fprintf(fout, "; creation-date=\"%s\"", cdsp->creation_date);
    }
    if (cdsp->modification_date) {
      fprintf(fout, "; modification-date=\"%s\"", cdsp->modification_date);
    }
    if (cdsp->read_date) {
      fprintf(fout, "; read-date=\"%s\"", cdsp->read_date);
    }
    if (cdsp->size > 0) {
      fprintf(fout, "; size=%d", cdsp->size);
    }
    if (cdsp->others) {
      fprintf(fout, "; %s", cdsp->others);
    }
    fputc(LF, fout);
  }
  if (multipart->content_description) {
    fprintf(fout, "Content-Description: %s\n", multipart->content_description);
  }
}

/*
 * Function Name:	printContentInfo
 * Description:		print MIME header fields to file
 * Arguments:		MailHeader pointer, output FILE pointer
 * Return Value:	none
 */
static void
printContentInfo(content_type, fout)
	ContentType *	content_type;
	FILE *		fout;
{
  if (content_type && content_type->media_type) {
    fprintf(fout, "Content-Type: %s", content_type->media_type);
    if (content_type->charset) {
      fprintf(fout, "; charset=%s", content_type->charset);
    }
    if (content_type->name) {
      fprintf(fout, "; name=\"%s\"", content_type->name);
    }
    if (content_type->others) {
      fprintf(fout, "; %s", content_type->others);
    }
    fputc(LF, fout);
  }
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	extractBodyOfMultiPart
 * Description:		extract the body of multipart mail
 * Arguments:		MultiPart pointer
 * Return Value:	none
 */
static void
extractBodyOfMultiPart(multipart)
	MultiPart *	multipart;
{
  register FILE *	fout = NULL;
  char			buf[BUFSIZ];

  /* multipart */
  for (; multipart != NULL; multipart = multipart->next) {
    conv_t	ret;
    String	filename = NULL;

    if (multipart->content_disposition) {
      ContentDisposition *	cdsp = multipart->content_disposition;

      if (cdsp->filename != NULL &&
	  (strcasecmp(cdsp->disposition, "attachment") == 0 ||
	   strcasecmp(cdsp->disposition, "inline") == 0)) {
	ret = wkfMimeStringToHexString(cdsp->filename, buf, sizeof(buf));
	if (ret != CONV_ERR) {
	  filename = buf;
	}
      }
    }
    else if (multipart->content_type) {
      if (multipart->content_type->name) {
	ret = wkfMimeStringToHexString(multipart->content_type->name,
				       buf, sizeof(buf));
	if (ret != CONV_ERR) {
	  filename = buf;
	}
      }
    }

    fout = fopen(filename ? filename : multipart->filename, "w");
    if (fout == NULL) {
      error();
    }

    /* decode base64'ed multipart body */
    if (multipart->content_transfer_encoding &&
	strcasecmp(multipart->content_transfer_encoding, "base64") == 0) {
      wkfDecodeBase64File(multipart->ftmp, fout);
    }
    else if (multipart->content_transfer_encoding &&
	strcasecmp(multipart->content_transfer_encoding,
		   "quoted-printable") == 0) {
      wkfDecodeQuotedPrintableFile(multipart->ftmp, fout);
    }
    else {
      memset(buf, 0, sizeof(buf));
      while (fgets(buf, sizeof(buf), multipart->ftmp) != NULL) {
	fputs(buf, fout);
	memset(buf, 0, sizeof(buf));
      }
    }
    fclose(fout);
  }
}

