/*****************************************************************************
 *
 * FILE:	dump.c
 * DESCRIPTION:	print the header of mail
 * DATE:	Thu, Jun  8 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2001 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: dump.c,v 1.5 2001/06/29 10:08:10 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <wkf.h>
#include "mailtools.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/
#define	CHECK_NULL(x)	((x) == NULL ? "---" : (x))
#define	CHECK_NULL2(x)	((x) == NULL ? "***" : (x))


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	dumpHeader
 * Description:		show the header of mail/multipart
 * Arguments:		Mail pointer
 * Return Value:	none
 */
void
dumpHeader(mail)
	Mail *	mail;
{
  MailHeader *	header	  = mail->header;
  MultiPart *	multipart = mail->multipart;
  Received *	rp	  = NULL;
  conv_t	ret;
  char		buf[BUFSIZ];

  /* print receive header field */
  for (rp = header->received; rp != NULL; rp = rp->next) {
    fprintf(stdout, "From:\t%s\n"
		    "By:\t%s\n"
		    "Via:\t%s\n"
		    "With:\t%s\n"
		    "ID:\t%s\n"
		    "For:\t%s\n"
		    "Date:\t%s\n"
		    CRLF,
		    CHECK_NULL(rp->recv_from),
		    CHECK_NULL(rp->recv_by),
		    CHECK_NULL(rp->recv_via),
		    CHECK_NULL(rp->recv_with),
		    CHECK_NULL(rp->recv_id),
		    CHECK_NULL(rp->recv_for),
		    CHECK_NULL(rp->recv_date));
  }

  /* print another header field */
  if (header->date) {
    fprintf(stdout, "Date:\t%s\n", header->date);
  }
  if (header->from) {
    ret = wkfDecodeMimeString(header->from, buf, sizeof(buf), KC_EUC);
    fprintf(stdout, "From:\t%s\n", ret == CONV_OK ? buf : header->from);
  }
  if (header->sender) {
    fprintf(stdout, "Sender:\t%s\n", header->sender);
  }
  if (header->reply_to) {
    fprintf(stdout, "Reply-To:\t%s\n", header->reply_to);
  }
  if (header->to) {
    ret = wkfDecodeMimeString(header->to, buf, sizeof(buf), KC_EUC);
    fprintf(stdout, "To:\t%s\n", ret == CONV_OK ? buf : header->to);
  }
  if (header->cc) {
    ret = wkfDecodeMimeString(header->cc, buf, sizeof(buf), KC_EUC);
    fprintf(stdout, "Cc:\t%s\n", ret == CONV_OK ? buf : header->cc);
  }
  if (header->bcc) {
    ret = wkfDecodeMimeString(header->bcc, buf, sizeof(buf), KC_EUC);
    fprintf(stdout, "Bcc:\t%s\n", ret == CONV_OK ? buf : header->bcc);
  }
  if (header->message_id) {
    fprintf(stdout, "Message-ID:\t%s\n", header->message_id);
  }
  if (header->in_reply_to) {
    fprintf(stdout, "In-Reply-To:\t%s\n", header->in_reply_to);
  }
  if (header->references) {
    fprintf(stdout, "References:\t%s\n", header->references);
  }
  if (header->subject) {
    ret = wkfDecodeMimeString(header->subject, buf, sizeof(buf), KC_EUC);
    fprintf(stdout, "Subject:\t%s\n", ret == CONV_OK ? buf : header->subject);
  }
  if (header->comments) {
    fprintf(stdout, "Comments:\t%s\n", header->comments);
  }
  if (header->keywords) {
    fprintf(stdout, "Keywords:\t%s\n", header->keywords);
  }
  if (header->return_path) {
    fprintf(stdout, "Return-Path:\t%s\n", header->return_path);
  }
  if (header->mime_version) {
    fprintf(stdout, "MIME-Version:\t%s\n", header->mime_version);
  }

  if (header->content_type) {
    fprintf(stdout, "Content-Type:\t%s\n", header->content_type->media_type);
    if (header->content_type->charset) {
      fprintf(stdout, "  charset=\t%s\n", header->content_type->charset);
    }
    if (header->content_type->boundary) {
      fprintf(stdout, "  boundary=\t%s\n", header->content_type->boundary);
    }
    if (header->content_type->name) {
      fprintf(stdout, "  name=\t%s\n", header->content_type->name);
    }
    if (header->content_type->others) {
      fprintf(stdout, "  others=\t%s\n", header->content_type->others);
    }
  }
  if (header->content_transfer_encoding) {
    fprintf(stdout, "Content-Transfer-Encoding:\t%s\n",
		    header->content_transfer_encoding);
  }
  if (header->content_id) {
    fprintf(stdout, "Content-ID:\t%s\n", header->content_id);
  }
  if (header->content_description) {
    ret = wkfDecodeMimeString(header->content_description, buf, sizeof(buf),
			      KC_EUC);
    fprintf(stdout, "Content-Description:\t%s\n",
	    ret == CONV_OK ? buf : header->content_description);
  }
  fputs(CRLF, stdout);

  /* print multipart header field */
  for (; multipart != NULL; multipart = multipart->next) {
    if (multipart->filename) {
      fprintf(stdout, "Filename: %s\n", multipart->filename);
    }
    if (multipart->content_type) {
      fprintf(stdout, "Content-Type:\t%s\n",
		      multipart->content_type->media_type);
      if (multipart->content_type->charset) {
	fprintf(stdout, "  charset=\t%s\n", multipart->content_type->charset);
      }
      if (multipart->content_type->boundary) {
	fprintf(stdout, "  boundary=\t%s\n", multipart->content_type->boundary);
      }
      if (multipart->content_type->name) {
	ret = wkfDecodeMimeString(multipart->content_type->name,
				  buf, sizeof(buf), KC_EUC);
	fprintf(stdout, "  name=\t%s\n",
		ret == CONV_OK ? buf : multipart->content_type->name);
      }
      if (multipart->content_type->others) {
	fprintf(stdout, "  others=\t%s\n", multipart->content_type->others);
      }
    }

    if (multipart->content_transfer_encoding) {
      fprintf(stdout, "Content-Transfer-Encoding:\t%s\n",
		      multipart->content_transfer_encoding);
    }
    if (multipart->content_description) {
      ret = wkfDecodeMimeString(multipart->content_description,
				buf, sizeof(buf), KC_EUC);
      fprintf(stdout, "Content-Description:\t%s\n",
	      ret == CONV_OK ? buf : multipart->content_description);
    }

    if (multipart->content_disposition) {
      fprintf(stdout, "Content-Disposition:\t%s\n",
		      multipart->content_disposition->disposition);
      if (multipart->content_disposition->filename) {
	ret = wkfDecodeMimeString(multipart->content_disposition->filename,
				  buf, sizeof(buf), KC_EUC);
	fprintf(stdout, "  filename=\t\t%s\n",
		ret == CONV_OK ? buf
			       : multipart->content_disposition->filename);
      }
      if (multipart->content_disposition->creation_date) {
	fprintf(stdout, "  creation_date=\t%s\n",
			multipart->content_disposition->creation_date);
      }
      if (multipart->content_disposition->modification_date) {
	fprintf(stdout, "  modification_date=\t%s\n",
			multipart->content_disposition->modification_date);
      }
      if (multipart->content_disposition->read_date) {
	fprintf(stdout, "  read_date=\t\t%s\n",
			multipart->content_disposition->read_date);
      }
      if (multipart->content_disposition->size) {
	fprintf(stdout, "  size=\t\t\t%d\n",
			multipart->content_disposition->size);
      }
      if (multipart->content_disposition->others) {
	fprintf(stdout, "  others=\t\t%s\n",
			multipart->content_disposition->others);
      }
    }
    fputs("---\n", stdout);
  }
}


/*****************************************************************************
 *****************************************************************************/

#if	DEBUG
/*
 * Function Name:	debug
 * Description:		show the header of mail/multipart
 * Arguments:		Mail pointer
 * Return Value:	none
 */
void
debug(mail)
	Mail *	mail;
{
  MailHeader *	header	  = mail->header;
  MailBody *	body	  = mail->body;
  MultiPart *	multipart = mail->multipart;
  Received *	rp	  = NULL;

  for (rp = header->received; rp != NULL; rp = rp->next) {
    fprintf(stdout, "From:\t%s\n"
		    "By:\t%s\n"
		    "Via:\t%s\n"
		    "With:\t%s\n"
		    "ID:\t%s\n"
		    "For:\t%s\n"
		    "Date:\t%s\n"
		    CRLF,
		    CHECK_NULL(rp->recv_from),
		    CHECK_NULL(rp->recv_by),
		    CHECK_NULL(rp->recv_via),
		    CHECK_NULL(rp->recv_with),
		    CHECK_NULL(rp->recv_id),
		    CHECK_NULL(rp->recv_for),
		    CHECK_NULL(rp->recv_date));
  }

  fprintf(stdout, "Date:\t%s\n"
		  "From:\t%s\n"
		  "Sender:\t%s\n"
		  "Reply-To:\t%s\n"
		  "To:\t%s\n"
		  "Cc:\t%s\n"
		  "Bcc:\t%s\n"
		  "Message-ID:\t%s\n"
		  "In-Reply-To:\t%s\n"
		  "References:\t%s\n"
		  "Subject:\t%s\n"
		  "Comments:\t%s\n"
		  "Keywords:\t%s\n"
		  "Return-Path:\t%s\n"
		  "MIME-Version:\t%s\n",
		  CHECK_NULL(header->date),
		  CHECK_NULL(header->from),
		  CHECK_NULL(header->sender),
		  CHECK_NULL(header->reply_to),
		  CHECK_NULL(header->to),
		  CHECK_NULL(header->cc),
		  CHECK_NULL(header->bcc),
		  CHECK_NULL(header->message_id),
		  CHECK_NULL(header->in_reply_to),
		  CHECK_NULL(header->references),
		  CHECK_NULL(header->subject),
		  CHECK_NULL(header->comments),
		  CHECK_NULL(header->keywords),
		  CHECK_NULL(header->return_path),
		  CHECK_NULL(header->mime_version));
  if (header->content_type != NULL) {
    fprintf(stdout, "Content-Type:\t%s\n"
		    "  charset=\t%s\n"
		    "  boundary=\t%s\n"
		    "  name=\t%s\n"
		    "  others=\t%s\n"
		    "Content-Transfer-Encoding:\t%s\n"
		    "Content-ID:\t%s\n"
		    "Content-Description:\t%s\n"
		    CRLF,
		    CHECK_NULL(header->content_type->media_type),
		    CHECK_NULL(header->content_type->charset),
		    CHECK_NULL(header->content_type->boundary),
		    CHECK_NULL(header->content_type->name),
		    CHECK_NULL(header->content_type->others),
		    CHECK_NULL(header->content_transfer_encoding),
		    CHECK_NULL(header->content_id),
		    CHECK_NULL(header->content_description));
  }

  for (; multipart != NULL; multipart = multipart->next) {
    fprintf(stdout, "Filename: %s\n", CHECK_NULL2(multipart->filename));
    if (multipart->content_type != NULL) {
      fprintf(stdout, "Content-Type:\t%s\n"
		      "  charset=\t%s\n"
		      "  boundary=\t%s\n"
		      "  name=\t%s\n"
		      "  others=\t%s\n",
		      CHECK_NULL2(multipart->content_type->media_type),
		      CHECK_NULL2(multipart->content_type->charset),
		      CHECK_NULL2(multipart->content_type->boundary),
		      CHECK_NULL2(multipart->content_type->name),
		      CHECK_NULL2(multipart->content_type->others));
    }
    fprintf(stdout, "Content-Transfer-Encoding:\t%s\n"
		    "Content-Description:\t%s\n",
		    CHECK_NULL2(multipart->content_transfer_encoding),
		    CHECK_NULL2(multipart->content_description));

    if (multipart->content_disposition != NULL) {
      fprintf(stdout, "Content-Disposition:\t%s\n"
		      "  filename=\t\t%s\n"
		      "  creation_date=\t%s\n"
		      "  modification_date=\t%s\n"
		      "  read_date=\t\t%s\n"
		      "  size=\t\t\t%d\n"
		      "  others=\t\t%s\n",
		      CHECK_NULL2(multipart->content_disposition->disposition),
		      CHECK_NULL2(multipart->content_disposition->filename),
		      CHECK_NULL2(multipart->content_disposition->creation_date),
		      CHECK_NULL2(multipart->content_disposition->modification_date),
		      CHECK_NULL2(multipart->content_disposition->read_date),
		      multipart->content_disposition->size,
		      CHECK_NULL2(multipart->content_disposition->others));
    }
    fputs("---\n", stdout);
  }
}
#endif	/* DEBUG */
