/*****************************************************************************
 *
 * FILE:	filter.c
 * DESCRIPTION:	filter
 * DATE:	Wed, Jun 20 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2001 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: filter.c,v 1.4 2001/07/05 08:34:57 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <signal.h>
#include <errno.h>
#include "mailtools.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and sructures
 *
 *****************************************************************************/
#define	MAX_RETRY	5
#define	DOCTYPE		"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
#define	CONTENTTYPE	"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=EUC-JP\">\n"

typedef enum {	/* index page type */
  PAGE_BEGINNING,
  PAGE_MIDDLE,
  PAGE_END
} page_t;


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static void		makeIndexPage(int);
static void		printIndicator(int, page_t, FILE *);

static void		unlock(int);
static void		lockRead(int);
static void		lockWrite(int);
static void		lockAppend(int);
static struct flock *	lockFile(short, short);


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	actionFilter
 * Description:		split multipart mail, decode MIME encoding and
 *			make HTML files through standard input
 * Arguments:		none
 * Return Value:	successful: 0, otherwise: -1
 */
int
actionFilter(void)
{
  Mail *		mail = NULL;
  struct passwd *	pwd  = NULL;
  struct stat		sbuf;
  char			buf[8];
  int			fd;
  int			cnt = 0;
  int			seq = 0;
  int			ret = -1;

  /* get user entry */
  pwd = getpwuid(geteuid());
  if (pwd == NULL) {
    goto fail;
  }

  /* change directory to home directory */
  if (chdir(pwd->pw_dir) == -1) {
    goto fail;
  }

  /* change current working directory to MailTools directory */
  if (chdir(MAILTOOLS_DIR) == -1) {
    if (errno == ENOENT) {
      if (mkdir(MAILTOOLS_DIR, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH) == -1) {
	goto fail;
      }
      /* change current working directory to MailTools directory again */
      if (chdir(MAILTOOLS_DIR) == -1) {
	goto fail;
      }
    }
  }

again:
#if	0
  fd = open(SEQUENCE_FILE, O_CREAT|O_RDWR|O_EXLOCK, S_IRUSR|S_IWUSR);
#endif
  fd = open(SEQUENCE_FILE, O_CREAT|O_RDWR, S_IRUSR|S_IWUSR);
  if (fd == -1) {
    switch (errno) {
      case EINTR:
      case EWOULDBLOCK:
	sleep(1);
	if (cnt++ > MAX_RETRY) {
	  goto fail;
	}
	goto again;
      default:
	goto fail;
    }
  }
  lockWrite(fd);
  if (fstat(fd, &sbuf) == -1) {
    goto error;
  }

  if (sbuf.st_size == 0) {
    seq = 1;
  }
  else {
    memset(buf, 0, sizeof(buf));
    if (read(fd, buf, sizeof(buf)) < 0) {
      goto error;
    }
    seq = atoi(buf);
    if (seq <= 0) {
      goto error;
    }
  }

  memset(buf, 0, sizeof(buf));
  sprintf(buf, "%05d", seq);
  mail = mailOpen(buf, OPT_FILTER);
  if (mail == NULL) {
    goto error;
  }
  actionHTML(mail);
  mailClose(mail);

  memset(buf, 0, sizeof(buf));
  sprintf(buf, "%d", ++seq);
  if (lseek(fd, (off_t)0L, SEEK_SET) == -1 || write(fd, buf, strlen(buf)) < 0) {
    goto error;
  }

  ret = 0;
error:
  unlock(fd);
  close(fd);

  /**/
  makeIndexPage(seq);

fail:
  return ret;
}


/*****************************************************************************
 *****************************************************************************/

static void
makeIndexPage(seq)
	int	seq;
{
#define	MAX_ITEMS	10	/* max items in a page */
  char		file[16];	/* indexXXXXX.html */
  char		tag[10];	/* tag.XXXXX */
  FILE *	fout = NULL;
  FILE *	fin  = NULL;
  int		page = 1;
  int		count;
  int		i;
  char		buf[BUFSIZ];
  size_t	size;

  for (count = 0, i = seq - 1; i > 0; i--) {
    if (count % MAX_ITEMS == 0) {
#if	HAVE_SNPRINTF
      (void)snprintf(file, sizeof(file), "index%05d.html", page);
#else
      (void)sprintf(file, "index%05d.html", page);
#endif	/* HAVE_SNPRINTF */
      fout = fopen(file, "w");
      if (fout == NULL) {
	error();
      }
      fputs(DOCTYPE "<html lang=\"ja\">\n" "<head>\n" CONTENTTYPE, fout);
      fprintf(fout, "<title>Mail Index(%d)</title>\n</head>\n", page);
      fputs("<body lang=\"ja\" bgcolor=\"#ffffff\">\n", fout);
      fprintf(fout, "<table summary=\"Mail Index(%d)\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n", page);
    }

    /**/
#if	HAVE_SNPRINTF
    (void)snprintf(tag, sizeof(tag), "tag.%05d", i);
#else
    (void)sprintf(tag, "tag.%05d", i);
#endif	/* HAVE_SNPRINTF */
    fin = fopen(tag, "r");
    if (fin == NULL) {
      fclose(fout);
      (void)unlink(file);
      error();
    }
    while (!feof(fin)) {
      size = fread(buf, sizeof(char), sizeof(buf), fin);
      fwrite(buf, sizeof(char), size, fout);
    }
    fclose(fin);
    count++;

    /**/
    if (count % MAX_ITEMS == 0) {
      fputs("</table>\n<hr>\n", fout);
      if (i > seq - MAX_ITEMS - 1) {
	printIndicator(page, PAGE_BEGINNING, fout);
      }
      else if (i > 0) {
	printIndicator(page, PAGE_MIDDLE, fout);
      }
      else {
	printIndicator(page, PAGE_END, fout);
      }
      fputs("</body>\n</html>\n", fout);
      fclose(fout);
      page++;
      count = 0;
    }
  }
  if (count < MAX_ITEMS) {
    fputs("</table>\n", fout);
    if (page > 1) {
      fputs("<hr>\n", fout);
      printIndicator(page, PAGE_END, fout);
    }
    fputs("</body>\n</html>\n", fout);
    fclose(fout);
  }
}

/*
 */
static void
printIndicator(page, type, fout)
	int	page;	/* current page */
	page_t	type;
	FILE *	fout;
{
  char	file[16];	/* indexXXXXX.html */

  fputs("<div align=\"center\">", fout);
  switch (type) {
    case PAGE_BEGINNING:
#if	HAVE_SNPRINTF
      (void)snprintf(file, sizeof(file), "index%05d.html", page + 1);
#else
      (void)sprintf(file, "index%05d.html", page + 1);
#endif	/* HAVE_SNPRINTF */
      fprintf(fout, "[<a href=\"%s\">Next</a>]", file);
      break;
    case PAGE_END:
#if	HAVE_SNPRINTF
      (void)snprintf(file, sizeof(file), "index%05d.html", page - 1);
#else
      (void)sprintf(file, "index%05d.html", page - 1);
#endif	/* HAVE_SNPRINTF */
      fprintf(fout, "[<a href=\"%s\">Prev</a>]", file);
      break;
    case PAGE_MIDDLE:
#if	HAVE_SNPRINTF
      (void)snprintf(file, sizeof(file), "index%05d.html", page - 1);
      fprintf(fout, "[<a href=\"%s\">Prev</a>]", file);
      (void)snprintf(file, sizeof(file), "index%05d.html", page + 1);
      fprintf(fout, "[<a href=\"%s\">Next</a>]", file);
#else
      (void)sprintf(file, "index%05d.html", page - 1);
      fprintf(fout, "[<a href=\"%s\">Prev</a>]", file);
      (void)sprintf(file, "index%05d.html", page + 1);
      fprintf(fout, "[<a href=\"%s\">Next</a>]", file);
#endif	/* HAVE_SNPRINTF */
      break;
  }
  fputs("</div>", fout);
}



/*****************************************************************************
 *****************************************************************************/

/*
 */
static void
unlock(fd)
	int	fd;
{
  fcntl(fd, F_SETLKW, lockFile(F_UNLCK, SEEK_SET));
}

/* shared lock for whole file */
static void
lockRead(fd)
	int	fd;
{
  fcntl(fd, F_SETLKW, lockFile(F_RDLCK, SEEK_SET));
}

/* exclusive lock for whole file */
static void
lockWrite(fd)
	int	fd;
{
  fcntl(fd, F_SETLKW, lockFile(F_WRLCK, SEEK_SET));
}

/*
 */
static void
lockAppend(fd)
	int	fd;
{
  fcntl(fd, F_SETLKW, lockFile(F_WRLCK, SEEK_END));
}

/*
 */
static struct flock *
lockFile(type, whence)
	short	type;
	short	whence;
{
  static struct flock	ret;

  ret.l_type	= type;
  ret.l_whence	= whence;
  ret.l_start	= 0;
  ret.l_len	= 0;
  ret.l_pid	= getpid();

  return &ret;
}
