/*****************************************************************************
 *
 * FILE:	html.c
 * DESCRIPTION:	print HTML text
 * DATE:	Mon, Jun 18 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2001 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: html.c,v 1.6 2001/06/29 10:08:10 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wkf.h>
#include "mailtools.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	htmlPrintHeader
 * Description:		print header fields to file
 * Arguments:		MailHeader pointer, FILE pointer
 * Return Value:	none
 */
void
htmlPrintHeader(header, fout)
	MailHeader *	header;
	FILE *		fout;
{
  conv_t	ret;
  char		buf[BUFSIZ];

  fputs("<table summary=\"Mail Header Field\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n", fout);
  if (header->date) {
    fputs("<tr><td><font color=\"#3366ff\">Date:</font></td><td><font color=\"#3366ff\">", fout);
    htmlPuts(header->date, fout);
    fputs("</font></td></tr>\n", fout);
  }
  if (header->from) {
    fputs("<tr><td><font color=\"#cc33cc\">From:</font></td><td><font color=\"#cc33cc\">", fout);
    ret = wkfDecodeMimeString(header->from, buf, sizeof(buf), KC_EUC);
    htmlPuts(ret == CONV_OK ? buf : header->from, fout);
    fputs("</font></td></tr>\n", fout);
  }
  if (header->to) {
    fputs("<tr><td><font color=\"#33cc00\">To:</font></td><td><font color=\"#33cc00\">", fout);
    ret = wkfDecodeMimeString(header->to, buf, sizeof(buf), KC_EUC);
    htmlPuts(ret == CONV_OK ? buf : header->to, fout);
    fputs("</font></td></tr>\n", fout);
  }
  if (header->cc) {
    fputs("<tr><td>Cc:</td><td>", fout);
    ret = wkfDecodeMimeString(header->cc, buf, sizeof(buf), KC_EUC);
    htmlPuts(ret == CONV_OK ? buf: header->cc, fout);
    fputs("</td></tr>\n", fout);
  }
  if (header->bcc) {
    fputs("<tr><td>Bcc:</td><td>", fout);
    ret = wkfDecodeMimeString(header->bcc, buf, sizeof(buf), KC_EUC);
    htmlPuts(ret == CONV_OK ? buf : header->bcc, fout);
    fputs("</td></tr>\n", fout);
  }
  if (header->message_id) {
    fputs("<tr><td>Message-ID:</td><td>", fout);
    htmlPuts(header->message_id, fout);
    fputs("</td></tr>\n", fout);
  }
  if (header->subject) {
    fputs("<tr><td><font color=\"#ff0000\">Subject:</font></td><td><font color=\"#ff0000\">", fout);
    ret = wkfDecodeMimeString(header->subject, buf, sizeof(buf), KC_EUC);
    htmlPuts(ret == CONV_OK ? buf : header->subject, fout);
    fputs("</font></td></tr>\n", fout);
  }
  fputs("</table>\n", fout);
}

/*
 * Function Name:	htmlPuts
 * Description:		print escaped string
 * Arguments:		string, output FILE pointer
 * Return Value:	none
 */
void
htmlPuts(str, fout)
	const String	str;
	register FILE *	fout;
{
  String	p   = NULL;
  String	buf = NULL;
  size_t	len;

  if (str == NULL) {
    return;
  }

  len = strlen(str) + 1;
  buf = (String)calloc(len, sizeof(char));
  if (buf == NULL) {
    error();
  }

  if (wkfConvertKanjiCodeOfString(KC_JIS, str,
				  KC_EUC, buf, len) == CONV_ERR) {
    error();
  }

  for (p = buf; *p != EOL; p++) {
    if (*p == LF) {
      fputs("<br>\n", fout);
    }
    else if (*p == '&') {
      fputs("&amp;", fout);
    }
    else if (*p == '<') {
      fputs("&lt;", fout);
    }
    else if (*p == '>') {
      fputs("&gt;", fout);
    }
    else if (*p == '"') {
      fputs("&quot;", fout);
    }
    else if (*p == SPC || *p == TAB) {
      fputs("&nbsp;", fout);
    }
    else {
      fputc((int)*p, fout);
    }
  }
  free((String)buf);
}

/*
 * Function Name:	htmlPutsAnchor
 * Description:		print escaped anchor string
 * Arguments:		string, output FILE pointer
 * Return Value:	none
 */
void
htmlPutsAnchor(str, fout)
	const String	str;
	register FILE *	fout;
{
  String	p = str;

  if (p == NULL) {
    return;
  }

  for (; *p != EOL; p++) {
    if (*p == ':') {
      fputs("%3A", fout);
    }
    else if (*p == '+') {
      fputs("%2B", fout);
    }
    else if (*p == '=') {
      fputs("%3D", fout);
    }
    else if (*p == '?') {
      fputs("%3F", fout);
    }
    else {
      fputc((int)*p, fout);
    }
  }
}

