/*****************************************************************************
 *
 * FILE:	mailtools.c
 * DESCRIPTION:	simple mail operation tools
 * DATE:	Thu, Jun  8 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2001 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: mailtools.c,v 1.8 2001/06/29 10:08:10 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include "mailtools.h"
#include "functions.h"
#include "version.h"


/*****************************************************************************
 *
 *	Program & Copyright
 *
 *****************************************************************************/
const char	Program[] = "mailtools";
const char	Copyright[] = "Copyright (C) 2001 Kouichi ABE (WALL)";


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static void	copyright(void);
static void	usage(void);
static void	trap(void);


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	error
 * Description:		error trap
 * Arguments:		none
 * Return Value:	none
 */
void
error(void)
{
  fputs("*** ERROR ***\n", stderr);
  exit(1);
}

/*
 * Function Name:	copyright
 * Description:		print copyright
 * Arguments:		none
 * Return Value:	none
 */
static void
copyright(void)
{
  fprintf(stderr,
	  "%s\n"
	  "Version %s\n"
	  "%s\n"
	  "%s\n",
	  SOFTWARE_NAME,
	  SOFTWARE_VERSION,
	  Copyright,
	  SOFTWARE_ADDRESS);
}

/*
 * Function Name:	usage
 * Description:		show the simple usage for mailtools
 * Arguments:		none
 * Return Value:	none
 */
static void
usage(void)
{
  copyright();
  fprintf(stderr,
	  "usage: "
	  "%s [ -split | -extract | -html | -filter | -print ] <mail>...\n\n",
	  Program);
  exit(1);
}

/*
 * Function Name:	trap
 * Description:		signal trap function
 * Arguments:		none
 * Return Value:	none
 */
static void
trap(void)
{
  fputs("*** SIGNAL ERROR ***\n", stderr);
  exit(1);
}

/*
 * MAIN FUNCTION
 */
int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  Mail *	mail	= NULL;
  option_t	option	= OPT_SPLIT;	/* default */
  int		i;

  /* addressing arguments */
  while (--argc > 0 && (*++argv)[0] == '-') {
    if (strcmp(*argv, "-split") == 0 || strncmp(*argv, "-s", 2) == 0) {
      option = OPT_SPLIT;
    }
    else if (strcmp(*argv, "-extract") == 0 || strncmp(*argv, "-e", 2) == 0) {
      option = OPT_EXTRACT;
    }
    else if (strcmp(*argv, "-html") == 0 || strncmp(*argv, "-h", 2) == 0) {
      option = OPT_HTML;
    }
    else if (strcmp(*argv, "-filter") == 0 || strncmp(*argv, "-f", 2) == 0) {
      option = OPT_FILTER;
    }
    else if (strcmp(*argv, "-print") == 0 || strncmp(*argv, "-p", 2) == 0) {
      option = OPT_PRINT;
    }
    else if (strcmp(*argv, "-version") == 0 || strncmp(*argv, "-v", 2) == 0) {
      copyright();
      exit(0);
    }
    else {
      usage();
    }
  }

  /* set signal trap */
  signal(SIGINT,  (void *)trap);
  signal(SIGQUIT, (void *)trap);
  signal(SIGTERM, (void *)trap);
  signal(SIGBUS,  (void *)trap);
  signal(SIGSEGV, (void *)trap);

  /* -filter option is especially */
  if (option == OPT_FILTER) {
    return actionFilter();
  }

  /* processing every file */
  if (argc > 0) {
    for (i = 0; i < argc; i++) {
      mail = mailOpen(argv[i], option);
      if (mail == NULL) {
	continue;
      }
      switch (option) {
	case OPT_SPLIT:
	  actionSplit(mail);
	  break;
	case OPT_EXTRACT:
	  actionExtract(mail);
	  break;
	case OPT_HTML:
	  actionHTML(mail);
	  break;
	case OPT_PRINT:
#if	DEBUG
	  debug(mail);
#else
	  dumpHeader(mail);
#endif	/* DEBUG */
	  break;
	default:
	  break;
      }
      mailClose(mail);
    }
  }
  else {
    usage();
  }

  return 0;
}

