/*****************************************************************************
 *
 * FILE:	mailtools.h
 * DESCRIPTION:	simple mail operation tools header
 * DATE:	Thu, Jun  8 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2001 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: mailtools.h,v 1.5 2001/06/23 15:57:18 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_mailtools_h
#define	_mailtools_h	1

#include <stdio.h>


/*****************************************************************************
 *
 *	define macros and structures
 *
 *****************************************************************************/
#define	MAILTOOLS_DIR	"MailTools"
#define	SEQUENCE_FILE	".mailtools_sequence"
#define	TAG_PREFIX	"tag."

#define	LF	'\012'
#define	CR	'\015'
#define	CRLF	"\015\012"
#define	SPC	'\040'
#define	TAB	'\011'
#define	EOL	'\0'

#define	MH_DATE		"Date:"
/**/
#define	MH_FROM		"From:"
#define	MH_SENDER	"Sender:"
#define	MH_REPLY_TO	"Reply-To:"
/**/
#define	MH_TO		"To:"
#define	MH_CC		"Cc:"
#define	MH_BCC		"Bcc:"
/**/
#define	MH_MESSAGE_ID	"Message-ID:"
#define	MH_IN_REPLY_TO	"In-Reply-To:"
#define	MH_REFERENCES	"References:"
/**/
#define	MH_SUBJECT	"Subject:"
#define	MH_COMMENTS	"Comments:"
#define	MH_KEYWORDS	"Keywords:"
/**/
#define	MH_RETURN_PATH	"Return-Path:"
#define	MH_RECEIVED	"Received:"
/**/
#define	MH_MIME_VERSION			"MIME-Version:"
#define	MH_CONTENT_TYPE			"Content-Type:"
#define	MH_CONTENT_TRANSFER_ENCODING	"Content-Transfer-Encoding:"
#define	MH_CONTENT_ID			"Content-ID:"
#define	MH_CONTENT_DESCRIPTION		"Content-Description:"
#define	MH_CONTENT_DISPOSITION		"Content-Disposition:"

#define	MP_MIXED	"multipart/mixed"
#define	MP_FORM_DATA	"multipart/form-data"

#define	length(x)	(sizeof(x)-1)

#define	TEMPLATE_FILE	"/tmp/mailtools.XXXXXX"


#ifndef	_STRING_T
#define	_STRING_T	1
typedef char *	String;
#endif	/* _STRING_T */

#ifndef	_BOOL_T
#define	_BOOL_T	1
typedef enum { FALSE, TRUE } bool_t;
#endif	/* _BOOL_T */

typedef enum {	/* option type */
  OPT_SPLIT,	/* -split */
  OPT_EXTRACT,	/* -extract */
  OPT_HTML,	/* -html */
  OPT_FILTER,	/* -filter */
  OPT_PRINT	/* -print */
} option_t;


/* Received: field (see RFC2821) */
typedef struct _received {
  String	recv_from;	/* From-domain */
  String	recv_by;	/* By-domain */
  String	recv_via;	/* Optional information */
  String	recv_with;	/* Optional information */
  String	recv_id;	/* Optional information */
  String	recv_for;	/* Optional information */
  String	recv_date;
  /**/
  struct _received *	next;
} Received;

/* Content-Type: field */
typedef struct _content_type {
  String	media_type;	/* type/subtype */
  String	charset;
  String	boundary;
  String	name;
  String	others;
} ContentType;

/* Mail header fields (see RFC2822) */
typedef struct _mailheader {
  /* Origination date field */
  String	date;		/* Date: date-time */
  /* Originator fields */
  String	from;		/* From: mailbox-list */
  String	sender;		/* Sender: mailbox */
  String	reply_to;	/* Reply-To: address-list */
  /* Destination address fields */
  String	to;		/* To: address-list */
  String	cc;		/* Cc: address-list */
  String	bcc;		/* Bcc: address-list */
  /* Identification fields */
  String	message_id;	/* Message-ID: msg-id */
  String	in_reply_to;	/* In-Reply-To: msg-id */
  String	references;	/* References: msg-id */
  /* Information fields */
  String	subject;	/* Subject: unstructed */
  String	comments;	/* Comments: unstructed */
  String	keywords;	/* Keywords: phrase */
#if	0
  /* Resent fields */
  String	resent_date;	/* Resent-Date: date-time */
  String	resent_from;	/* Resent-From: mailbox-list */
  String	resent_sender;	/* Resent-Sender: mailbox */
  String	resent_to;	/* Resent-To: address-list */
  String	resent_cc;	/* Resent-Cc: address-list */
  String	resent_bcc;	/* Resent-Bcc: address-list */
  String	resent_msg_id;	/* Resent-Message-ID: msg-id */
#endif
  /* Trace fields */
  String	return_path;	/* Return-Path: path */
  Received *	received;	/* Received: name-val-list; date-time */
  /* MIME header fields (see RFC2045) */
  String	mime_version;	/* MIME-Version: version */
  ContentType *	content_type;	/* Content-Type: type/subtype; charset */
  String	content_transfer_encoding;
				/* Content-Transfer-Encoding: mechanism */
				/*
				 * mechanism := 7bit / 8bit / binary /
				 *		quoted-printable / base64
				 */
  String	content_id;	/* Content-ID: msg-id */
  String	content_description;	/* Content-Description: text */
} MailHeader;

/* Content-Disposition field (see RFC2183) */
typedef struct _content_disposition {
  String	disposition;	/* "inline" / "attachment" */
  String	filename;	/* "filename="value */
  String	creation_date;	/* "creation-date="quoted-date-time */
  String	modification_date;/* "modification-date="quoted-date-time */
  String	read_date;	/* "read-date="quoted-date-time */
  int		size;		/* "size="digit */
  String	others;
} ContentDisposition;

/* Multipart header fields (see RFC2045) */
typedef struct _multipart {
  String	filename;	/* filename on UNIX filesystem */
  FILE *	fout;
  String	tmpfile;	/* temporary file */
  FILE *	ftmp;
  /**/
  ContentType *	content_type;	/* Content-Type: type/subtype; name */
  String	content_transfer_encoding;
				/* Content-Transfer-Encoding: mechanism */
				/*
				 * mechanism := 7bit / 8bit / binary /
				 *		quoted-printable / base64
				 */
  ContentDisposition *	content_disposition;
				/* Content-Disposition: disposition-type */
  String	content_description;	/* Content-Description: text */
  /**/
  struct _multipart *	next;
} MultiPart;

typedef struct _mailbody {
  String	filename;	/* filename on UNIX filesystem */
  FILE *	fout;
  String	tmpfile;	/* temporary file */
  FILE *	ftmp;
} MailBody;

typedef struct _mail {
  String	filename;	/* filename on UNIX filesystem */
  FILE *	fin;
  int		parts;		/* number of multipart bodies */
  option_t	option;		/* command option */
  MailHeader *	header;
  MailBody *	body;
  MultiPart *	multipart;
} Mail;

#endif	/* _mailtools_h */
