/*****************************************************************************
 *
 * FILE:	util.c
 * DESCRIPTION:	useful functions for simple mail operation library
 * DATE:	Thu, Jun  8 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2001 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: util.c,v 1.4 2001/06/23 15:57:19 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include "mailtools.h"

#ifndef	HAVE_MKSTEMP
/*
 * Function Name:	mkstemp
 * Description:		create temporary file and return its file descriptor
 * Arguments:		template path
 * Return Value:	file descriptor
 */
int
mkstemp(path)
	String	path;
{
  String	p = mktemp(path);

  if (p == NULL) {
    return -1;
  }

  return open(path, O_CREAT|O_EXCL|O_RDWR, 0600);
}
#endif	/* HAVE_MKSTEMP */

/*
 * Function Name:	append
 * Description:		append two strings
 * Arguments:		source string, destination string
 * Return Value:	appended string
 */
String
append(src, dst)
	String	src;
	String	dst;
{
  size_t	len;
  String	str = NULL;

  if (src == NULL || dst == NULL) {
    return dst;
  }

  len = strlen(src) + strlen(dst) + 1 + 1;
  str = (String)calloc(len, sizeof(char));
  if (str == NULL) {
    return dst;
  }
#if	HAVE_SNPRINTF
  (void)snprintf(str, len, "%s %s", dst, src);
#else
  (void)sprintf(str, "%s %s", dst, src);
#endif	/* HAVE_SNPRINTF */
  free((String)dst); dst = NULL;

  return str;
}

/*
 * Function Name:	chomp
 * Description:		remove CRLF, CR or LF from string end
 * Arguments:		string
 * Return Value:	string
 */
String
chomp(str)
	String	str;
{
  String	p = NULL;
  size_t	len;

  if (str == NULL) {
    return NULL;
  }

  len = strlen(str);
  p   = &str[len - 1];
  if (*p == LF) {
    *p-- = EOL;
  }
  if (*p == CR) {
    *p-- = EOL;
  }

  return str;
}

/*
 * Function Name:	xstrdup
 * Description:		save a copy of a string
 * Arguments:		string
 * Return Value:	same as strdup()
 */
String
xstrdup(str)
	String	str;
{
  if (str == NULL) {
    return str;
  }
  return strdup(str);
}
